/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.editor.Editor');

/**
 * @constant
 * @type {boolean}
 * Is tiny mce initiated ?
 */
org.ametys.cms.editor.Editor._tinyMCEInitiated = false;

/**
 * @private
 */
org.ametys.cms.editor.Editor._tags = {};
/**
 * This method retrieve a tag to handle.
 * Just by calling this methodd, the tag will be handle by the editor.
 * Once you have a tag it a map
 * - emptyTag Can be empty, +, - or #. see tinymce valid_elements documentation
 * - getAttributes Call this method to handle an attribute. This will return a map where key is the attribute name and value are
 *          - values An array of possible values
 *          - defaultValue The default value. Can be null if there is no
 * - replaceTags An array of tags that will be replaced by this one. This value can be dynamically removed if some one register this other tag as handled.
 * 
 * e.g. var aTag = org.ametys.cms.editor.Editor.handleTag("a"); // from now <a> will be accepted
 *      aTag.emptyTag = "+"; // The <a> tag will be forced opened
 *      var classAttr = aTag.handleAttribute("class"); // The class attribute is now accepted on the <a> tag
 *      classAttr.defaultValue = "myclass"; // The class attribute will now always exists on <a> tag with the value "myclass"
 *      classAttr.values.push("myclass");
 *      classAttr.values.push("myclass2"); // The class attribute will now accept 2 values myclass or myclass2
 */
org.ametys.cms.editor.Editor.handleTag = function(tagName)
{
	if (org.ametys.cms.editor.Editor._tags[tagName] == null)
	{
		org.ametys.cms.editor.Editor._tags[tagName] = {
				tagName: tagName,
				emptyTag: "",
				attributes: {},
				handleAttribute: function(attributeName)
				{
					if (this.attributes[attributeName] == null)
					{
						this.attributes[attributeName] = {
							attributeName: attributeName,
							values: [],
							defaultValue: null,
							toString: function() {
								var a = this.defaultValue != null ? this.attributeName + "=" + this.defaultValue : "";

								var b = ""
								for (var key in this.values)
								{
									if (typeof this.values[key] == "string")
									{
										if (b != "") 
										{
											b += "?";
										}
																		
										b += this.values[key];
									}
								}
								if (b != "")
								{
									b = this.attributeName + "<" + b;
								}
								
								if (a == "" && b == "")
								{
									return this.attributeName;
								}
								else 
								{
									return a + ((a != "" && b != "") ? "|" : "") + b;
								}
							}
						};
					}
					return this.attributes[attributeName];
				},
				replaceTags: [],
				toString: function() {
					var base = this.emptyTag + tagName;
					
					var attributes = "";
					for (var key in this.attributes)
					{
						if (attributes != "")
						{
							attributes += "|"; 
						}
						attributes += this.attributes[key].toString(); 
					}
					
					var replaceTags = "";
					for (var i = 0; i < this.replaceTags.length; i++)
					{
						// The second part of the condition allow to remove multiple instance of the same string
						if (org.ametys.cms.editor.Editor._tags[this.replaceTags[i]] == null && this.replaceTags.indexOf(this.replaceTags[i]) == i)
						{
							replaceTags += "/" + this.replaceTags[i];
						}
					}
					
					return base + (attributes == "" ? "" : "[" + attributes + "]") + replaceTags;
				}
		};
	}
	return org.ametys.cms.editor.Editor._tags[tagName];
}
org.ametys.cms.editor.Editor.handleTag("p").emptyTag = "#";
org.ametys.cms.editor.Editor.handleTag("br");
var divTag = org.ametys.cms.editor.Editor.handleTag("div");
divTag.handleAttribute('class');
divTag.handleAttribute('id');

/**
 * A list of css files to load in the inline editor
 */
org.ametys.cms.editor.Editor.useCSSFile = [];
org.ametys.cms.editor.Editor.useCSSFile.push(getPluginResourcesUrl('cms') + "/css/content/ContentToolEditor.css");

/**
 * Storing conversion methods
 * onSetContent is an array of method with signature (editor, object) that is called when the editor received new content. This allows to convert storing tags to internal tags. Use object.content to get/set the full html.
 * onGetContent is an array of method with signature (editor, object) that is called when the editor received content. This allows to convert internat tags to storing tags. Use object.content to get/set the full html.
 */
org.ametys.cms.editor.Editor.onSetContent = [];
org.ametys.cms.editor.Editor.onGetContent = [];

/**
 * @static
 * Initiate the tiny mce. Before calling this method set the tags yoo handle through the getTag method
 * @param cssFiles An optionnal list of css files to load as a comma-separated string. eg: "/plugins/cms/resources/css/mycss.css;/plugins/cms/resources/css/mycss2.css". This parameter is used only at the first call of this method
 */
org.ametys.cms.editor.Editor.init = function()
{
	if (org.ametys.cms.editor.Editor._tinyMCEInitiated)
	{
		return;
	}
	
	var cssFilesToLoad = "";
	for (var i = 0; i < org.ametys.cms.editor.Editor.useCSSFile.length; i++)
	{
		cssFilesToLoad += org.ametys.cms.editor.Editor.useCSSFile[i] + ",";
	}
	
	var validElements = "";
	for (var key in org.ametys.cms.editor.Editor._tags)
	{
		if (validElements != "")
		{
			validElements += ",";
		}
		validElements += org.ametys.cms.editor.Editor._tags[key].toString();
	}

	tinyMCE.init
	({
		// General options
		mode : "none",
		theme : "advanced",
		plugins : "table,paste,tabfocus,noneditable",
		doctype: "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">",

		entity_encoding : 'raw',
		fix_list_elements : true,
		fix_table_elements : true,
		fix_nesting : true,
		verify_css_classes : false,
		gecko_spellcheck : true,
		valid_elements : validElements,
		paste_strip_class_attributes: "mso",
		paste_remove_styles: true,
		strict_loading_mode : true,

		// Theme options
		theme_advanced_buttons1 : "",
		theme_advanced_buttons2 : "",
		theme_advanced_buttons3 : "",
	
		// Theme options
		theme_advanced_toolbar_location : "none",
		theme_advanced_statusbar_location : "none",
		
		content_css : cssFilesToLoad,
		
		setup: function(ed)
		{
			if (Ext.isIE)
			{
				ed.onEvent.add(tinyMCE.save);
			}

			ed.onNodeChange.add (org.ametys.cms.editor.Editor._onRichTextNodeSelected);
			ed.onBeforeSetContent.add(org.ametys.cms.editor.Editor._onBeforeSetContent);
			ed.onGetContent.add(org.ametys.cms.editor.Editor._onGetContent);
		}
	});
	tinyMCE.save = function()
	{
		if (Ext.isIE)
		{
			tinyMCE.activeEditor._bookmark = tinyMCE.activeEditor.selection.getBookmark(1);
		}
	}
	tinyMCE.focus = function ()
	{
		tinyMCE.activeEditor.focus();
		if (tinyMCE.activeEditor._bookmark)
		{
			tinyMCE.activeEditor.selection.moveToBookmark(tinyMCE.activeEditor._bookmark);
		}

	}
	org.ametys.cms.editor.Editor._tinyMCEInitiated = true;
}

/**
 * @private
 * When setting content  in the rich text
 */
org.ametys.cms.editor.Editor._onBeforeSetContent = function (editor, object)
{
	for (var i = 0; i < org.ametys.cms.editor.Editor.onSetContent.length; i++)
	{
		org.ametys.cms.editor.Editor.onSetContent[i](editor, object);
	}
}
	
/**
 * @private
 * When getting content  in the rich text
 */
org.ametys.cms.editor.Editor._onGetContent = function (editor, object)
{
	for (var i = 0; i < org.ametys.cms.editor.Editor.onGetContent.length; i++)
	{
		org.ametys.cms.editor.Editor.onGetContent[i](editor, object);
	}
}

/**
 * @private
 * When a node is selected in the rich text
 */
org.ametys.cms.editor.Editor._onRichTextNodeSelected = function(editor, controlManager, node, isCollapsed)
{
	if (editor.notFirstCallToOnRichTextNodeSelected)
	{
		var cmp = Ext.getCmp(editor.id); 
		var contentTool = cmp.contentTool;
		contentTool.onHTMLNodeSelected.defer(1, contentTool, [cmp.name, node]);
	}
	else
	{
		editor.onClick.remove(editor.nodeChanged);
	}
	editor.notFirstCallToOnRichTextNodeSelected = true;
}

/**
 * Insert the html given at the root of the current selection of the current editor
 * @param {String} html The html to insert
 * @param {String} mode The mode 'split' : to split the current place of the cursor to insert the html ; 'after' to inster after the root element of the current selection ; 'before' idem but before. 'split' is the default value
 */
org.ametys.cms.editor.Editor.insertHTMLAtRoot = function(html, mode)
{
	mode = mode || 'split';
	
	var rootelements = 'h1,h2,h3,h4,h5,h6,p';
	
	var root = tinyMCE.activeEditor.dom.getParent(tinyMCE.activeEditor.selection.getNode(), rootelements);

	if (mode == 'split' || root == null)
	{
		// UPGRADE tinyMCE 3.3.3 plugins/table/js/table.js insertTable
		tinyMCE.activeEditor.selection.setContent('<br class="_mce_marker" />');
	
		var patt = '';
		tinymce.each(rootelements.split(','), function(n) {
			if (patt)
				patt += ',';
	
			patt += n + ' ._mce_marker';
		});
	
		tinymce.each(tinyMCE.activeEditor.dom.select(patt), function(n) {
			tinyMCE.activeEditor.dom.split(tinyMCE.activeEditor.dom.getParent(n, rootelements), n);
		});
	
		tinyMCE.activeEditor.dom.setOuterHTML(tinyMCE.activeEditor.dom.select('br._mce_marker')[0], html);
	}
	else
	{
		var br = tinyMCE.activeEditor.dom.doc.createElement("br")
		root.parentNode.insertBefore(br, mode == 'after' ? root.nextSibling : root);
		tinyMCE.activeEditor.dom.setOuterHTML(br, html);
	}
}
